using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Newtonsoft.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Collections;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class DepartmentController : Controller {
        private readonly ILogger<DepartmentController> _logger;
        private readonly FMSContext _context;
        private readonly DepartmentService _service;
        private readonly StorageHelper _storage;

        public DepartmentController(
            ILogger<DepartmentController> logger,
            FMSContext context,
            DepartmentService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// 그룹 목록
        /// </summary>
        /// <param name="req"></param>
        /// <param name="siteId"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req, [FromQuery] string siteId) {
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var paging = PagingList.Pagination(list, req.page, req.limit);
            return Ok(paging);
        }

        /// <summary>
        /// 조회
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("{siteId}/{companyId}/{id}")]
        public IActionResult Get(int siteId, int companyId, int id) {
            var data = _service.Gets(id, siteId, companyId);
            if (data.Count() == 0) {
                return NotFound("부서정보를 찾을 수 없습니다.");
            }
            var item = data.Select(x => new {
                x.DepartmentId,
                x.SiteId,
                SiteName = x.Site.Name,
                CmSite = new {
                    x.Site.Name,
                },
                x.CompanyId,
                CompanyName = x.CmCompany.Name,
                CmCompany = new {
                    x.CmCompany.Name,
                },
                x.Name,
                x.ShortName,
                x.FaxNo,
                x.PhoneNo,
                x.Comment,
                x.IsUse,
            });

            return Ok(item.First());
        }

        /// <summary>
        /// 엑셀파일 익스포트
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            req.page = 1;
            req.limit = 0;
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 수정
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPost("{siteId}/{companyId}/{id}")]
        public IActionResult Edit(int siteId, int companyId, int id, [FromBody] CmDepartment data) {
            if (id != data.DepartmentId) {
                return BadRequest("잘못된 접근입니다.");
            }

            if (ModelState.IsValid) {
                _service.Edit(id, siteId, companyId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 등록
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPut]
        public IActionResult Create([FromBody] CmDepartment data) {
            _logger.LogInformation("Create");
            //_logger.LogInformation(data.Dump());

            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        [HttpDelete("{siteId}/{companyId}/{id}")]
        public IActionResult Delete(int siteId, int companyId, int id) {
            _service.Delete(id, siteId, companyId);

            return Ok();
        }

        // 셀렉트 공통
        private dynamic _ListSelect(IQueryable<CmDepartment> query) {
            var list = query.Select(x => new {
                x.SiteId,
                SiteName = x.Site.Name,
                x.DepartmentId,
                x.CompanyId,
                CompanyName = x.CmCompany.Name,
                x.Name,
                x.PhoneNo,
                IsUse = x.IsUse.GetValueOrDefault() ? "Y" : "N",
            });
            return list;
        }

        // 검색 & 정렬 공통
        private IQueryable<CmDepartment> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();

            // 기본 Entity 검색
            query = query.Filter(req.conditions);

            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }

            // 고정정렬 (사옥 준공순)
            var sortQuery = query.OrderBy(x => x.Site.SortOrderNo);

            // 추가 정렬
            sortQuery = sortQuery.ThenSort(req.sort);

            return sortQuery;
        }
    }
}