using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Newtonsoft.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Collections;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class EquipmentHistoryController : Controller {
        private readonly ILogger<EquipmentHistoryController> _logger;
        private readonly FMSContext _context;
        private readonly EquipmentHistoryService _service;
        private readonly StorageHelper _storage;

        public EquipmentHistoryController(
            ILogger<EquipmentHistoryController> logger,
            FMSContext context,
            EquipmentHistoryService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }


        /// <summary>
        /// 공기구 등록 목록
        /// </summary>
        [HttpGet("[action]")]
        public IActionResult AddStockList([FromQuery] PagingRequest req) {
            return List(req, 1);
        }


        /// <summary>
        /// 공기구 대여 목록
        /// </summary>
        [HttpGet("[action]")]
        public IActionResult RentList([FromQuery] PagingRequest req) {
            return List(req, 2);
        }

        /// <summary>
        /// 공기구 반납/손실 목록
        /// </summary>
        [HttpGet("[action]")]
        public IActionResult ReturnList([FromQuery] PagingRequest req) {
            return List(req, 3);
        }

        /// <summary>
        /// 공기구 등록 목록
        /// </summary>
        [HttpGet("[action]")]
        public IActionResult ExcelAddStockList([FromQuery] PagingRequest req, string filename) {
            return ExportExcel(req, 1, filename);
        }

        /// <summary>
        /// 공기구 반납/손실 목록
        /// </summary>
        [HttpGet("[action]")]
        public IActionResult ExcelReturnList([FromQuery] PagingRequest req, string filename) {
            return ExportExcel(req, 3, filename);
        }


        /// <summary>
        /// 리스트
        /// </summary>
        /// <param name="req"></param>
        /// <param name="siteId"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req, int? stateType) {
            var query = _FilterAndSort(req, stateType);

            if (stateType != null) {
                if (stateType == 2) { // 대여 
                    query = query.Where(x => x.EquipmentStateTypeId != 1 && x.IsReturned == false);
                } else if (stateType == 3) {
                    query = query.Where(x => x.EquipmentStateTypeId == 3 || x.EquipmentStateTypeId == 4 || x.EquipmentStateTypeId == 5);
                } else {
                    query = query.Where(x => x.EquipmentStateTypeId == stateType);
                }
            } else {
                if (!_context.SiteConfig.Single().UseEquipmentRent) {
                    query = query.Where(x => x.EquipmentStateTypeId != 2);
                }
            }

            var list = _ListSelect(query);

            var paging = PagingList.Pagination(list, req.page, req.limit);
            return Ok(paging);
        }

        /// <summary>
        /// 엑셀파일 익스포트
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, int? stateType, string filename) {
            var query = _FilterAndSort(req, stateType);
            if (stateType != null) {
                if (stateType == 2) { // 대여 
                    query = query.Where(x => x.EquipmentStateTypeId != 1 && x.IsReturned == false);
                } else if (stateType == 3) {
                    query = query.Where(x => x.EquipmentStateTypeId == 3 || x.EquipmentStateTypeId == 4 || x.EquipmentStateTypeId == 5);
                } else {
                    query = query.Where(x => x.EquipmentStateTypeId == stateType);
                }
            } else {
                if (!_context.SiteConfig.Single().UseEquipmentRent) {
                    query = query.Where(x => x.EquipmentStateTypeId != 2);
                }
            }


            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 조회
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Gets(id, siteId);
            if (data.Count() == 0) {
                return NotFound("공기구정보를 찾을 수 없습니다.");
            }
            var item = data.Select(x => new {
                x.SiteId,
                x.EquipmentId,
                x.EquipmentHistoryId,
                x.EquipmentRentId,
                Site = new {
                    x.CmSite.Name,
                },
                EquipmentStateType = new {
                    x.EquipmentStateType.Name
                },
                RentInfo = new {
                    Title = x.FmsEquipmentRentInfo.Title,
                    RentUserName = x.FmsEquipmentRentInfo.RentUserName,
                    RentDate = x.FmsEquipmentRentInfo.RentDate,
                    ReturnDueDate = x.FmsEquipmentRentInfo.ReturnDueDate,
                    ReturnFixDate = x.FmsEquipmentRentInfo.ReturnFixDate
                },
                EquipmentInfo = new {
                    x.FmsEquipment.Name,
                    Standard = x.FmsEquipment.Standard,
                },
                EquipmentName = x.FmsEquipment.Name,
                x.RentCount,
                x.CurrentRentCount,
                x.LossCount,
                x.ReturnCount,
                x.ReturnReason,
                x.IsReturnApproval,
                ReturnApprovalDate = x.ReturnApprovalDate != null ? x.ReturnApprovalDate.Value.ToString("yyyy/MM/dd") : "",
                ReturnApprovalUser = x.CmUser.Name,
                CmFile = _storage.ParseFile(x.CmFile),
            });

            return Ok(item.First());
        }


        /// <summary>
        /// 수정
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPost("{siteId}/{id}")]
        public IActionResult Edit(int siteId, int id, [FromBody] FmsEquipmentHistory data) {
            if (id != data.EquipmentId) {
                return BadRequest("잘못된 접근입니다.");
            }
            ModelState.Remove("RegisterUserId");
            if (ModelState.IsValid) {
                _service.Edit(id, siteId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 등록
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPut]
        public IActionResult Create([FromBody] FmsEquipmentHistory data) {
            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpDelete("{siteId}/{id}")]
        public IActionResult Delete(int siteId, int id) {
            _service.Delete(id, siteId);

            return Ok();
        }

        // 셀렉트 공통
        private dynamic _ListSelect(IQueryable<FmsEquipmentHistory> query) {
            var list = query.Select(x => new {
                x.EquipmentHistoryId,
                x.EquipmentId,
                x.SiteId,
                SiteName = x.CmSite.Name,
                EquipmentStateTypeName = x.EquipmentStateType.Name,
                EquipmentTypeId = x.FmsEquipment.EquipmentTypeId,
                EquipmentTypeName = x.FmsEquipment.FmsEquipmentCodeType.Name,
                EquipmentName = x.FmsEquipment.Name,
                Standard = x.FmsEquipment.Standard,
                Unit = x.FmsEquipment.Unit,
                x.TotalStockCount,
                x.StoredCount,
                x.ReturnCount,
                x.LossCount,
                x.RentCount,
                x.CurrentRentCount,
                x.CurrentStockCount,
                x.ReturnReason,
                AddDate = x.AddDate != null ? x.AddDate.Value.ToString("yyyy/MM/dd") : null,
                SupplierName = x.FmsEquipment.SupplierName,
                SupplierPhoneNo = x.FmsEquipment.SupplierPhoneNo,
                x.EquipmentRentId,
                x.EquipmentStateTypeId,
                RentUserName = x.FmsEquipmentRentInfo.RentUserName,
                RentDate = x.FmsEquipmentRentInfo.RentDate != null ? x.FmsEquipmentRentInfo.RentDate.Value.ToString("yyyy/MM/dd") : null,
                ReturnDueDate = x.FmsEquipmentRentInfo.ReturnDueDate != null ? x.FmsEquipmentRentInfo.ReturnDueDate.Value.ToString("yyyy/MM/dd") : null,
                ReturnFixDate = x.FmsEquipmentRentInfo.ReturnFixDate != null ? x.FmsEquipmentRentInfo.ReturnFixDate.Value.ToString("yyyy/MM/dd") : null,
                RegisterUserId = x.FmsEquipment.RegisterUserId,
                WarehouseId = x.FmsEquipment.WarehouseId,
                WarehouseName = x.FmsEquipment.FmsMaterialWarehouse.Name,
                ReturnApprovalUserName = x.ReturnApprovalUserId != null ? x.CmUser.Name : "",
                ReturnApprovalDate = x.ReturnApprovalDate != null ? x.ReturnApprovalDate.Value.ToString("yyyy/MM/dd") : null,
            });
            return list;
        }

        // 검색 & 정렬 공통
        private IQueryable<FmsEquipmentHistory> _FilterAndSort(PagingRequest req, int? stateType) {
            var query = _service.GetAll();

            // 기본 Entity 검색
            query = query.Filter(req.conditions);

            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }

            if (Util.IsNullOrEmpty(stateType)) {
                // 고정정렬 (사옥 준공순)
                var sortQuery = query.OrderBy(x => x.CmSite.SortOrderNo);
                // 추가 정렬
                sortQuery = sortQuery.ThenSort(req.sort);
                return sortQuery;
            } else {
                if (stateType == 1) {
                    var sortQuery = query.OrderByDescending(x => x.EquipmentId);
                    sortQuery = sortQuery.ThenByDescending(x => x.EquipmentHistoryId);
                    // 추가 정렬
                    sortQuery = sortQuery.ThenSort(req.sort);
                    return sortQuery;
                } else {
                    // 기본 Entity 정렬
                    query = query.Sort(req.sort);
                    return query;
                }
            }
        }

        [HttpGet("[action]")]
        public IActionResult ReturnApproval([FromQuery] int siteId, [FromQuery] int id, [FromQuery] string userId) {
            _service.ReturnApproval(siteId, id, userId);

            return Ok();
        }

    }
}