using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Newtonsoft.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Collections;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class EquipmentRentInfoController : Controller {
        private readonly ILogger<EquipmentRentInfoController> _logger;
        private readonly EquipmentRentInfoService _service;
        private readonly StorageHelper _storage;

        public EquipmentRentInfoController(
            ILogger<EquipmentRentInfoController> logger,
            EquipmentRentInfoService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// 리스트
        /// </summary>
        /// <param name="req"></param>
        /// <param name="siteId"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req, int? stateType) {
            var query = _FilterAndSort(req);
            if (stateType != null)
                query = query.Where(x => x.EquipmentStateTypeId == stateType);

            var list = _ListSelect(query);

            var paging = PagingList.Pagination(list, req.page, req.limit);
            return Ok(paging);
        }

        /// <summary>
        /// 엑셀파일 익스포트
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, int? stateType, string filename) {
            req.page = 1;
            req.limit = 0;
            var query = _FilterAndSort(req);
            if (stateType != null)
                query = query.Where(x => x.EquipmentStateTypeId == stateType);

            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 엑셀파일 임포트
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("[action]")]
        public IActionResult ImportExcel(IFormFile file) {
            IList<Hashtable> list = new List<Hashtable>();
            using (ExcelPackage excelPackage = new ExcelPackage(file.OpenReadStream())) {
                var worksheet = excelPackage.Workbook.Worksheets[0];
                for (int i = worksheet.Dimension.Start.Row; i <= worksheet.Dimension.End.Row; i++) {

                    var name = worksheet.Cells[i, 1].Value;
                    var rfid = worksheet.Cells[i, 2].Value;
                    var code = worksheet.Cells[i, 3].Value;
                    list.Add(new Hashtable {
                        {"Name", name}
                    });
                }
            }
            //_service.ImportExcel(list);
            //return BadRequest(ex.Message);

            return Ok();
        }

        /// <summary>
        /// 조회
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Gets(id, siteId);
            if (data.Count() == 0) {
                return NotFound("공기구대여정보를 찾을 수 없습니다.");
            }
            var item = data.Select(x => new {
                x.EquipmentRentId,
                x.EquipmentStateTypeId,
                FmsEquipmentCodeStateType = new {
                    x.EquipmentStateType.Name
                },
                x.RentDate,
                x.ReturnDueDate,
                x.SiteId,
                Site = new {
                    x.Site.Name,
                },
                x.Title,
                x.Comment,
                x.RentUserName,
                EquipmentHistory = x.FmsEquipmentHistory.Where(x => x.EquipmentStateTypeId == 2 && x.CurrentRentCount > 0).Select(h => new {
                    h.EquipmentHistoryId,
                    h.EquipmentId,
                    Name = h.FmsEquipment.Name,
                    RentCount = h.CurrentRentCount,
                    Standard = h.FmsEquipment.Standard,
                    WarehouseName = h.FmsEquipment.FmsMaterialWarehouse.Name
                }),
            });

            return Ok(item.First());
        }

        /// <summary>
        /// 수정
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPost("{siteId}/{id}")]
        public IActionResult Edit(int siteId, int id, [FromBody] FmsEquipmentRentInfo data) {
            if (id != data.EquipmentRentId) {
                return BadRequest("잘못된 접근입니다.");
            }
            ModelState.Remove("RegisterUserId");
            if (ModelState.IsValid) {
                _service.Edit(id, siteId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }


        /// <summary>
        /// 공기구 반납/손실 
        /// </summary>
        [HttpPost("[action]")]
        public IActionResult EditReturnInfo(int siteId, int historyId, [FromBody] FmsEquipmentRentInfo data) {

            ModelState.Remove("RegisterUserId");
            ModelState.Remove("Title");
            if (ModelState.IsValid) {
                _service.CreateReturnInfo(historyId, siteId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 등록
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPut]
        public IActionResult Create([FromBody] FmsEquipmentRentInfo data) {
            //_logger.LogInformation(data.Dump());

            // 현장ID설정            
            foreach (var history in data.FmsEquipmentHistory) {
                history.SiteId = data.SiteId;
                _logger.LogInformation("SiteId:" + history.SiteId);
                _logger.LogInformation("EquipmentId:" + history.EquipmentId);
            }
            ModelState.Remove("RegisterUserId");
            data.RegisterUserId = User.Identity.Name;

            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpDelete("{siteId}/{id}")]
        public IActionResult Delete(int siteId, int id) {
            _service.Delete(siteId, id);

            return Ok();
        }

        // 셀렉트 공통
        private dynamic _ListSelect(IQueryable<FmsEquipmentRentInfo> query) {
            var list = query.Select(x => new {
                x.EquipmentRentId,
                x.SiteId,
                SiteName = x.Site.Name,
                x.EquipmentStateTypeId,
                EquipmentStateTypeName = x.EquipmentStateType.Name,
                x.Title,
                x.Comment,
                x.RentUserName,
                RentDate = x.RentDate != null ? x.RentDate.Value.ToString("yyyy/MM/dd") : "",
                ReturnDueDate = x.ReturnDueDate != null ? x.ReturnDueDate.Value.ToString("yyyy/MM/dd") : ""
            });
            return list;
        }

        // 검색 & 정렬 공통
        private IQueryable<FmsEquipmentRentInfo> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();

            // 기본 Entity 검색
            query = query.Filter(req.conditions);

            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }


            // 기본 Entity 정렬
            query = query.Sort(req.sort);

            return query;
        }

    }
}