using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Newtonsoft.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Collections;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class MaterialPurchaseOrderController : Controller {
        private readonly ILogger<MaterialPurchaseOrderController> _logger;
        private readonly FMSContext _context;
        private readonly MaterialPurchaseOrderService _service;
        private readonly StorageHelper _storage;

        public MaterialPurchaseOrderController(
            ILogger<MaterialPurchaseOrderController> logger,
            FMSContext context,
            MaterialPurchaseOrderService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// 리스트
        /// </summary>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req, int? progressId = null) {
            var query = _FilterAndSort(req, progressId);

            var list = _ListSelect(query);

            var paging = PagingList.Pagination(list, req.page, req.limit);
            return Ok(paging);
        }

        /// <summary>
        /// 엑셀파일 익스포트
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            req.page = 1;
            req.limit = 0;
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 엑셀파일 임포트
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("[action]")]
        public IActionResult ImportExcel(IFormFile file) {
            IList<Hashtable> list = new List<Hashtable>();
            using (ExcelPackage excelPackage = new ExcelPackage(file.OpenReadStream())) {
                var worksheet = excelPackage.Workbook.Worksheets[0];
                for (int i = worksheet.Dimension.Start.Row; i <= worksheet.Dimension.End.Row; i++) {

                    var name = worksheet.Cells[i, 1].Value;
                    var rfid = worksheet.Cells[i, 2].Value;
                    var code = worksheet.Cells[i, 3].Value;
                    list.Add(new Hashtable {
                        {"Name", name}
                    });
                }
            }
            //_service.ImportExcel(list);
            //return BadRequest(ex.Message);

            return Ok();
        }

        /// <summary>
        /// 조회
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Gets(id, siteId);
            if (data.Count() == 0) {
                return NotFound("자재 발주 목록을 찾을 수 없습니다.");
            }
            var item = data.Select(x => new {
                x.PurchaseOrderId,
                x.SiteId,
                CmSite = new {
                    x.Site.Name,
                },
                x.BusinessFieldId,
                CmBusinessField = new {
                    x.CmBusinessField.Name,
                },
                x.PurchaseRequestId,
                RequestTitle = x.FmsMaterialPurchaseRequest.Title,
                x.TradeCompanyId,
                x.OrderDate,
                x.OrderUserId,
                OrderUserName = x.CmUser.Name,
                x.Title,
                x.Reason,
                x.StoredDueDate,
                ProgressId = x.FmsMaterialPurchaseRequest.ProgressId,
                ProgressName = x.FmsMaterialPurchaseRequest.Progress.Name,
                RequestReason = x.FmsMaterialPurchaseRequest.Reason,
                MaterialPurchaseOrderMaterial = x.FmsMaterialPurchaseOrderMaterial.Select(f => new {
                    f.SiteId,
                    f.BusinessFieldId,
                    f.PurchaseOrderId,
                    f.MaterialId,
                    f.MaterialCount,
                    f.UnitCost,
                    Name = f.FmsMaterial.Name,
                    Standard = f.FmsMaterial.Standard,
                    Unit = f.FmsMaterial.Unit,
                    FinalPrice = f.UnitCost,
                    f.FmsMaterial.FirstClassId,
                    FirstClassName = f.FmsMaterial.FmsMaterialCodeClass.Name,
                    f.FmsMaterial.SecondClassId,
                    SecondClassName = f.FmsMaterial.FmsMaterialCodeClass1.Name,
                    f.FmsMaterial.ThirdClassId,
                    ThirdClassName = f.FmsMaterial.FmsMaterialCodeClass2.Name,
                }).ToList(),
            });

            return Ok(item.First());
        }

        /// <summary>
        /// 수정
        /// </summary>
        [HttpPost("{siteId}/{id}")]
        public IActionResult Edit(int siteId, int id, [FromBody] FmsMaterialPurchaseOrder data) {
            if (id != data.PurchaseOrderId) {
                return BadRequest("잘못된 접근입니다.");
            }

            if (ModelState.IsValid) {
                _service.Edit(id, siteId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 등록
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPut]
        public IActionResult Create([FromBody] FmsMaterialPurchaseOrder data) {
            _logger.LogInformation("Create");
            //_logger.LogInformation(data.Dump());

            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpDelete("{siteId}/{id}")]
        public IActionResult Delete(int siteId, int id) {
            _service.Delete(id, siteId);

            return Ok();
        }

        // 셀렉트 공통
        private dynamic _ListSelect(IQueryable<FmsMaterialPurchaseOrder> query) {
            var list = query.Select(x => new {
                x.PurchaseOrderId,
                x.SiteId,
                SiteName = x.Site.Name,
                x.BusinessFieldId,
                BusinessFieldName = x.CmBusinessField.Name,
                ProgressName = x.FmsMaterialPurchaseRequest.Progress.Name,
                x.Title,
                x.OrderUserId,
                OrderUserName = x.CmUser.Name,
                OrderDate = (x.OrderDate != null) ? x.OrderDate.ToString("yyyy-MM-dd") : "",
                StoredDueDate = (x.StoredDueDate.Value != null) ? x.StoredDueDate.Value.ToString("yyyy-MM-dd") : "",
                x.PurchaseRequestId,
                x.TradeCompanyId,
                TradeCompanyName = x.FmsMaterialTradeCompany.Name,
                x.Reason,
                MaterialPurchaseOrderMaterial = x.FmsMaterialPurchaseOrderMaterial.Select(f => new {
                    f.SiteId,
                    f.PurchaseOrderId,
                    f.MaterialId,
                    f.MaterialCount,
                    UnitCost = f.FmsMaterial.FinalPrice,
                    Name = f.FmsMaterial.Name,
                    Standard = f.FmsMaterial.Standard,
                    Unit = f.FmsMaterial.Unit,
                    FinalPrice = f.UnitCost,
                    f.FmsMaterial.FirstClassId,
                    FirstClassName = f.FmsMaterial.FmsMaterialCodeClass.Name,
                    f.FmsMaterial.SecondClassId,
                    SecondClassName = f.FmsMaterial.FmsMaterialCodeClass1.Name,
                    f.FmsMaterial.ThirdClassId,
                    ThirdClassName = f.FmsMaterial.FmsMaterialCodeClass2.Name,
                    PossibleStoredCount = f.MaterialCount - Util.ToInt(_context.FmsMaterialStored.Where(fs => fs.SiteId == f.SiteId && fs.PurchaseOrderId == f.PurchaseOrderId && fs.MaterialId == f.MaterialId).Sum(fs => fs.StoredCount)),      //입고가능수량
                    //StoredCount = _context.FmsMaterialStored.Where(fs => fs.SiteId == f.SiteId && fs.PurchaseOrderId == f.PurchaseOrderId && fs.MaterialId == f.MaterialId).Select(fs => fs.StoredCount).FirstOrDefault(),     //입고수량
                    StoredCount = f.MaterialCount - Util.ToInt(_context.FmsMaterialStored.Where(fs => fs.SiteId == f.SiteId && fs.PurchaseOrderId == f.PurchaseOrderId && fs.MaterialId == f.MaterialId).Sum(fs => fs.StoredCount)),      //입고수량
                }).ToList(),
            });
            return list;
        }

        // 검색 & 정렬 공통
        private IQueryable<FmsMaterialPurchaseOrder> _FilterAndSort(PagingRequest req, int? progressId = null) {
            var query = _service.GetAll();

            // 기본 Entity 검색
            query = query.Filter(req.conditions);

            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }

            // 업무분야 필드가 있는경우만 추가
            if (req.businessFieldId != null) {
                query = query.Where(x => x.SiteId == req.siteId);
                query = query.Where(x => x.BusinessFieldId == req.businessFieldId || x.CmBusinessField.Name == "공통");
            }

            if (progressId != null) {
                query = query.Where(x => x.FmsMaterialPurchaseRequest.ProgressId == progressId);
            }

            // 고정정렬 (사옥 준공순)
            var sortQuery = query.OrderBy(x => x.Site.SortOrderNo);

            // 추가 정렬
            sortQuery = sortQuery.ThenSort(req.sort);

            return sortQuery;
        }
    }
}