using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Newtonsoft.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Collections;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class MaterialReleaseController : Controller {
        private readonly ILogger<MaterialReleaseController> _logger;
        private readonly FMSContext _context;
        private readonly MaterialReleaseService _service;
        private readonly StorageHelper _storage;

        public MaterialReleaseController(
            ILogger<MaterialReleaseController> logger,
            FMSContext context,
            MaterialReleaseService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// 리스트
        /// </summary>
        /// <param name="req"></param>
        /// <param name="siteId"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req) {
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var paging = PagingList.Pagination(list, req.page, req.limit);
            return Ok(paging);
        }

        /// <summary>
        /// 엑셀파일 익스포트
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            req.page = 1;
            req.limit = 0;
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 엑셀파일 임포트
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("[action]")]
        public IActionResult ImportExcel(IFormFile file) {
            IList<Hashtable> list = new List<Hashtable>();
            using (ExcelPackage excelPackage = new ExcelPackage(file.OpenReadStream())) {
                var worksheet = excelPackage.Workbook.Worksheets[0];
                for (int i = worksheet.Dimension.Start.Row; i <= worksheet.Dimension.End.Row; i++) {

                    var name = worksheet.Cells[i, 1].Value;
                    var rfid = worksheet.Cells[i, 2].Value;
                    var code = worksheet.Cells[i, 3].Value;
                    list.Add(new Hashtable {
                        {"Name", name}
                    });
                }
            }
            //_service.ImportExcel(list);
            //return BadRequest(ex.Message);

            return Ok();
        }

        /// <summary>
        /// 조회
        /// </summary>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Gets(id, siteId);
            if (data.Count() == 0) {
                return NotFound("자재 정산 목록을 찾을 수 없습니다.");
            }
            var item = data.Select(x => new {
                x.MaterialReleaseId,
                x.SiteId,
                CmSite = new {
                    x.Site.Name,
                },
                x.BusinessFieldId,
                CmBusinessField = new {
                    x.CmBusinessField.Name,
                },
                x.AdjustmentTypeId,
                AdjustmentTypeName = x.AdjustmentType.Name,
                x.AdjustmentUserId,
                AdjustmentUserName = x.CmUser.Name,
                x.ConfirmedUserId,
                ConfirmedUserName = x.CmUserNavigation.Name,
                x.MaterialId,
                x.ReleaseTypeId,
                ReleaseTypeName = x.ReleaseType.Name,
                x.Comment,
                x.IsConfirmed,
                Materials = data.Select(f => new {
                    f.MaterialReleaseId,
                    f.MaterialId,
                    MaterialName = f.FmsMaterial.Name,
                    f.FmsMaterial.FirstClassId,
                    FirstClassName = f.FmsMaterial.FmsMaterialCodeClass.Name,
                    f.FmsMaterial.SecondClassId,
                    SecondClassName = f.FmsMaterial.FmsMaterialCodeClass1.Name,
                    f.FmsMaterial.ThirdClassId,
                    ThirdClassName = f.FmsMaterial.FmsMaterialCodeClass2.Name,
                    StockCount = f.StockCountByPoint,
                    f.StockCountByPoint,
                    f.MaterialCount,
                }).ToList(),
            });

            return Ok(item.First());
        }

        /// <summary>
        /// 수정
        /// </summary>
        [HttpPost("{siteId}/{id}")]
        public IActionResult Edit(int siteId, int id, [FromBody] FmsMaterialRelease data) {
            if (id != data.MaterialReleaseId) {
                return BadRequest("잘못된 접근입니다.");
            }

            if (ModelState.IsValid) {
                _service.Edit(id, siteId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 등록
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPut]
        public IActionResult Create([FromBody] FmsMaterialRelease data) {
            _logger.LogInformation("Create");
            //_logger.LogInformation(data.Dump());

            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        [HttpDelete("{siteId}/{id}")]
        public IActionResult Delete(int siteId, int id) {
            _service.Delete(id, siteId);

            return Ok();
        }

        // 셀렉트 공통
        private dynamic _ListSelect(IQueryable<FmsMaterialRelease> query) {
            var list = query.Select(x => new {
                x.MaterialReleaseId,
                x.SiteId,
                SiteName = x.Site.Name,
                x.BusinessFieldId,
                BusinessFieldName = x.CmBusinessField.Name,
                x.AdjustmentTypeId,
                AdjustmentTypeName = x.AdjustmentType.Name,
                x.FmsMaterial.FirstClassId,
                FirstClassName = x.FmsMaterial.FmsMaterialCodeClass.Name,
                x.FmsMaterial.SecondClassId,
                SecondClassName = x.FmsMaterial.FmsMaterialCodeClass1.Name,
                x.FmsMaterial.ThirdClassId,
                ThirdClassName = x.FmsMaterial.FmsMaterialCodeClass2.Name,
                MaterialName = x.FmsMaterial.Name,
                MaterialStandard = x.FmsMaterial.Standard,
                MaterialUnit = x.FmsMaterial.Unit,
                x.MaterialCount,
                ReleaseDate = (x.ReleaseDate != null) ? x.ReleaseDate.ToString("yyyy-MM-dd") : "",
                x.AdjustmentUserId,
                AdjustmentUserName = x.CmUser.Name,
                x.ConfirmedUserId,
                ConfirmedUserName = x.CmUserNavigation.Name,
                ConfirmedDate = (x.ConfirmedDate != null) ? Convert.ToDateTime(x.ConfirmedDate).ToString("yyyy-MM-dd") : "",
                x.MaterialId,
                x.IsConfirmed,
            });
            return list;
        }

        // 검색 & 정렬 공통
        private IQueryable<FmsMaterialRelease> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();

            // 기본 Entity 검색
            query = query.Filter(req.conditions);

            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }

            // 업무분야 필드가 있는경우만 추가
            if (req.businessFieldId != null) {
                query = query.Where(x => x.SiteId == req.siteId);
                query = query.Where(x => x.BusinessFieldId == req.businessFieldId || x.CmBusinessField.Name == "공통");
            }

            query = query.Where(x => x.AdjustmentTypeId != 0);

            // 고정정렬 (사옥 준공순)
            var sortQuery = query.OrderBy(x => x.Site.SortOrderNo);

            // 추가 정렬
            if (req.sort.field == "ReleaseAndConfirmed") { // 기본 커스텀 정렬 예제 (최근 정산일순)
                sortQuery = sortQuery.ThenByDescending(x => x.ReleaseDate);
            } else {
                sortQuery = sortQuery.ThenSort(req.sort);
            }

            return sortQuery;
        }


        /// <summary>
        /// 승인
        /// </summary>
        [HttpPost("{userId}")]
        public IActionResult Approval(string userId, [FromBody] ICollection<FmsMaterialRelease> data) {
            //if (ModelState.IsValid) {
            if (data.Count > 0) {
                _service.Approval(userId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 개별 승인
        /// </summary>
        [HttpGet("[action]")]
        public IActionResult ApprovalEach([FromQuery] int siteId, [FromQuery] int id, [FromQuery] string userId) {
            _service.ApprovalEach(siteId, id, userId);

            return Ok();
        }
    }
}