using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class TenantController : Controller {
        private readonly ILogger<TenantController> _logger;
        private readonly FMSContext _context;
        private readonly TenantService _service;
        private readonly StorageHelper _storage;

        public TenantController(
            ILogger<TenantController> logger,
            FMSContext context,
            TenantService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// 검색 & 정렬
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        private IQueryable<CmTenant> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();
            query = query.Filter(req.conditions);
            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }

            // 고정정렬 (사옥 준공순)
            var sortQuery = query.OrderBy(x => x.CmSite.SortOrderNo);

            // 추가 정렬
            sortQuery = sortQuery.ThenSort(req.sort);

            return sortQuery;
        }

        /// <summary>
        /// 셀렉트 공통
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        private dynamic _ListSelect(IQueryable<CmTenant> query) {
            var list = query.Select(x => new {
                x.SiteId,
                SiteName = x.CmSite.Name,
                x.TenantId,
                x.FloorId,
                FloorName = x.CmFloor.Name,
                SortNum = x.CmFloor.CmFloorCode.SortNum,
                x.Name,
                AreaSize = AreaParse(x.AreaSize),
                ExclusiveArea = AreaParse(x.ExclusiveArea),
                x.RepresentativePhone,
                x.CompanyTypeId,
                CompanyTypeName = x.CmCompanyType.Name,
                x.RepresentativeName,
                x.Fax,
                x.ChargeName,
                x.ChargePhone,
                x.AddressZip,
                x.Address1,
                x.Address2,
                x.Homepage,
                x.Comment,
                x.ChargeEmail,
                IsUse = x.IsUse.GetValueOrDefault() ? "Y" : "N",
                x.BuildingId,
                BuildingName = x.CmBuilding.Name

            });
            return list;
        }

        private static string AreaParse(String areaVal) {
            string returnVal = "";
            float val;
            if (!Util.IsNullOrEmpty(areaVal) && float.TryParse(areaVal, out val)) {
                returnVal = val.ToString("#,##0.00") + "㎡ (" + (val / 3.3058).ToString("#,##0.00") + "평)";
            }
            return returnVal;
        }
        /// <summary>
        /// 목록
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req) {
            var query = _FilterAndSort(req);
            var list = _ListSelect(query);
            var paging = PagingList.Pagination(list, req.page, req.limit);

            return Ok(paging);
        }

        // 엑셀 다운로드
        [AllowAnonymous]
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            var query = _FilterAndSort(req);
            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }


        /// <summary>
        /// 조회
        /// </summary>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Get(id, siteId);
            if (data.Count() == 0) {
                return NotFound("사이트 정보를 찾을 수 없습니다.");
            }

            var list = data.Select(x => new {
                x.SiteId,
                x.TenantId,
                x.CompanyTypeId,
                x.BuildingId,
                x.FloorId,
                x.Name,
                x.RepresentativeName,
                x.RepresentativePhone,
                x.Fax,
                AreaSize = x.AreaSize,
                x.ChargeName,
                x.ChargePhone,
                x.AddressZip,
                x.Address1,
                x.Address2,
                x.Comment,
                x.Homepage,
                x.IsUse,
                x.ChargeEmail,
                ExclusiveArea = x.ExclusiveArea,
                CmCompanyType = new {
                    x.CmCompanyType.Name
                },
                CmSite = new {
                    x.CmSite.Name,
                },
                CmBuilding = new {
                    x.CmBuilding.Name
                },
                CmFloor = new {
                    x.CmFloor.Name
                },
            });
            return Ok(list.First());
        }


        /// <summary>
        /// 수정
        /// </summary>
        [HttpPost("{siteId}/{id}")]
        public IActionResult Edit(int siteId, int id, [FromBody] CmTenant data) {
            _logger.LogInformation("Edit");
            _logger.LogInformation("id:" + id);
            _logger.LogInformation("siteId:" + siteId);
            //_logger.LogInformation(data.Dump());
            if (id != data.TenantId) {
                return BadRequest("잘못된 접근입니다.>>");
            }

            if (ModelState.IsValid) {
                _service.Edit(id, siteId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 등록
        /// </summary>
        [HttpPut]
        public IActionResult Create([FromBody] CmTenant data) {
            _logger.LogInformation("Create");
            //_logger.LogInformation(data.Dump());

            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }


        [HttpPost]
        [Route("[action]")]
        public IActionResult ImportExcelAdmin(IFormFile excelFile) {
            return (ImportExcel(excelFile, true));
        }

        /// <summary>
        /// 엑셀파일 임포트
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("[action]")]
        public IActionResult ImportExcel(IFormFile excelFile, bool isAdmin = false) {
            IList<CmTenant> list = new List<CmTenant>();
            var userInfo = _context.CmUser.Where(x => x.UserId == User.Identity.Name).SingleOrDefault();

            using (ExcelPackage excelPackage = new ExcelPackage(excelFile.OpenReadStream())) {
                var worksheet = excelPackage.Workbook.Worksheets[0];
                int colCnt = isAdmin ? 18 : 17;
                ExcelUploadHepler excelUploadHelper = new ExcelUploadHepler();
                //헤더세팅
                excelUploadHelper.GenerateColumns(worksheet, 1, colCnt);

                for (int i = worksheet.Dimension.Start.Row + 1; i <= worksheet.Dimension.End.Row; i++) {
                    if (!excelUploadHelper.IsRowEmpty(worksheet, i, colCnt)) {
                        var site = isAdmin ? excelUploadHelper.GetValue(worksheet, "사옥(*)", i, true) : "";
                        var floor = excelUploadHelper.GetValue(worksheet, "층 명칭(*)", i, true);
                        var name = excelUploadHelper.GetValue(worksheet, "회사 명칭(*)", i, true);
                        var areaSize = excelUploadHelper.GetValue(worksheet, "임대면적", i);
                        var exclusiveArea = excelUploadHelper.GetValue(worksheet, "전용면적", i);
                        var representativeName = excelUploadHelper.GetValue(worksheet, "회사 대표(*)", i, true);
                        var representativePhone = excelUploadHelper.GetValue(worksheet, "대표 번호(*)", i, true);
                        var companyType = excelUploadHelper.GetValue(worksheet, "회사 구분(*)", i, true);
                        var fax = excelUploadHelper.GetValue(worksheet, "팩스 번호", i);
                        var chargeName = excelUploadHelper.GetValue(worksheet, "담당자 명", i);
                        var chargePhone = excelUploadHelper.GetValue(worksheet, "담당자 전화", i);
                        var chargeEmail = excelUploadHelper.GetValue(worksheet, "담당자 이메일", i);
                        var addresszip = excelUploadHelper.GetValue(worksheet, "우편 번호", i);
                        var address1 = excelUploadHelper.GetValue(worksheet, "회사주소1", i);
                        var address2 = excelUploadHelper.GetValue(worksheet, "회사주소2", i);
                        var homepage = excelUploadHelper.GetValue(worksheet, "홈페이지", i);
                        var comment = excelUploadHelper.GetValue(worksheet, "비고", i);
                        var building = excelUploadHelper.GetValue(worksheet, "빌딩 명칭(*)", i, true);
                        _logger.LogInformation("site " + site);
                        _logger.LogInformation("type " + companyType);
                        _logger.LogInformation("codnamee " + name);

                        var rowName = "(" + (i) + "번 행)";

                        var siteId = isAdmin ? _context.CmSite.Where(x => x.Name == site && x.IsUse == true).Select(x => x.SiteId).SingleOrDefault() : userInfo.SiteId;
                        var buildingId = _context.CmBuilding.Where(x => x.Name == building && x.SiteId == siteId).Select(x => x.BuildingId).SingleOrDefault();
                        var floorId = _context.CmFloor.Where(x => x.Name == floor && x.BuildingId == buildingId && x.SiteId == siteId).Select(x => x.FloorId).SingleOrDefault();
                        var companyTypeId = _context.CmCompanyType.Where(x => x.Name == companyType/* && x.SiteId == siteId*/).Select(x => x.CompanyTypeId).SingleOrDefault();

                        if (isAdmin) excelUploadHelper.EntityIdCheck(siteId, "사옥", rowName);
                        excelUploadHelper.EntityIdCheck(buildingId, "빌딩명칭", rowName);
                        excelUploadHelper.EntityIdCheck(floorId, "층명칭", rowName);
                        excelUploadHelper.EntityIdCheck(companyTypeId, "회사구분", rowName);
                        if (!Util.IsNullOrEmpty(areaSize)) excelUploadHelper.FloatCheck(areaSize, "임대면적", rowName);
                        if (!Util.IsNullOrEmpty(exclusiveArea)) excelUploadHelper.FloatCheck(exclusiveArea, "전용면적", rowName);

                        var tenant = new CmTenant {
                            SiteId = siteId,
                            BuildingId = buildingId,
                            FloorId = floorId,
                            CompanyTypeId = companyTypeId,
                            Name = name,
                            RepresentativeName = representativeName,
                            RepresentativePhone = representativePhone,
                            IsUse = true,
                            Fax = fax,
                            ChargeName = chargeName,
                            ChargePhone = chargePhone,
                            ChargeEmail = chargeEmail,
                            AreaSize = areaSize,
                            ExclusiveArea = exclusiveArea,
                            AddressZip = addresszip,
                            Address1 = address1,
                            Address2 = address2,
                            Homepage = homepage,
                            Comment = comment,
                            ExcelRowNum = i
                        };
                        // 글자수 체크
                        TryValidateModel(tenant);

                        if (!ModelState.IsValid) throw new ServiceException(string.Join(" ",
                                        ModelState.Values
                                        .SelectMany(x => x.Errors)
                                        .Select(x => x.ErrorMessage)) + rowName);

                        //중복 체크
                        var overlapRowNum = list.Where(x => x.SiteId == tenant.SiteId
                                                    && x.BuildingId == tenant.BuildingId
                                                    && x.FloorId == tenant.FloorId
                                                    && x.Name == tenant.Name).Select(x => x.ExcelRowNum).SingleOrDefault();

                        if (overlapRowNum != 0) {
                            throw new ServiceException("파일 내 동일 정보가 존재합니다.[빌딩명칭-층명칭-회사명칭]" + "(" + (overlapRowNum) + "번 행 - " + (i) + "번 행)");
                        }

                        if (_service.checkName(tenant, true) > 0) {
                            throw new ServiceException("기존 입주사와 동일 정보가 존재합니다.[빌딩명칭-층명칭-회사명칭]" + rowName);
                        }

                        // 데이터 값 설정
                        list.Add(tenant);
                    }
                }
            }
            if (list.Count > 0) {
                _service.ImportExcel(list);

                return Ok($"엑셀업로드를 성공하였습니다. {list.Count}건의 데이터가 등록되었습니다.");
            } else {
                throw new ServiceException("등록할 데이터가 없습니다.");
            }
        }

        /// <summary>
        /// 삭제
        /// </summary>
        [HttpDelete("{siteId}/{id}")]
        public IActionResult Delete(int siteId, int id) {
            _service.Delete(id, siteId);
            return Ok();
        }



    }
}